body {{ 
    color: #000;
    background-color: #fff;
}}
::-webkit-scrollbar {{
    background: silver;
}}
::-webkit-scrollbar-thumb {{
    background: gray;
}}
::-webkit-scrollbar-thumb:hover {{
    background: #494949;
}}
::-webkit-scrollbar-corner {{
    background: {QTMATERIAL_SECONDARYLIGHTCOLOR};
}}
p {{ 
    color: #000;
}}
a {{
    color: #2b4e67 !important;
}}
a:hover {{
    color: #426f90 !important;
}}
.msg-user {{
    background-color: #f3f3f3;
    color: #000;
}}
.msg-user p {{
    color: #000;
}}
.msg-bot {{
    color: #000;
}}
.msg-bot p {{
    color: #000;
}}
.cmd {{
    color: #888787 !important;
    text-decoration: none;
}}
.ts {{
    color: #4d4d4d;
}}
code {{
    color: #000;
}}
.code-wrapper {{
    color: #fff;
    border: 1px solid #d4d4d4;
}}
.code-header-wrapper {{
    color: #fff;
    background: #e1e1e1;
}}
.code-header-action {{
    color: #444444 !important;
}}
.code-header-action:hover {{
    color: #000 !important;
    cursor: pointer;
}}
.code-header-action:hover img {{
    filter: brightness(60%);
}}
.code-header-lang {{
    color: #5c5c5c;
}}
.code-wrapper .code-header-action.code-header-copy:hover, 
.code-wrapper .code-header-action.code-header-collapse:hover {{
    transform: scale(1.06);
    background: var(--copy-btn-bg-hover, rgb(225 225 225 / 76%));
    border-color: var(--copy-btn-border, rgba(0, 0, 0, .08));
}}
.name-bot::before {{
    background-color: #6b6b6b;
}}
.name-user::before {{
    background-color: silver;
}}
.action-icons a:hover {{
    filter: brightness(60%);
}}
.tool-output .content {{
    color: #888787 !important;
}}

/* loader */
.lds-ring {{
   color: #000;
   opacity:0.5;
}}
.append_live {{
  color: gray;
}}

.tips p {{
 color: #6e6e6e;
}}
.msg-box .name-header.name-bot {{
    color: #7a787c;
}}